package org.eclipse.pde.http.ui;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IVMRunner;
import org.eclipse.jdt.launching.VMRunnerConfiguration;
import org.eclipse.jst.server.tomcat.core.internal.TomcatLaunchConfigurationDelegate;
import org.eclipse.pde.ui.launcher.EquinoxLaunchConfiguration;
import org.eclipse.wst.server.core.IServer;
import org.eclipse.wst.server.core.ServerUtil;
import org.eclipse.wst.server.core.internal.Server;

public class EquinoxHttpLaunchConfiguration extends EquinoxLaunchConfiguration {
	@SuppressWarnings("restriction")
	public void launch(ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {
		super.launch(configuration, mode, launch, monitor);
		IServer server = ServerUtil.getServer(configuration);
		if (server.canPublish().isOK()) {
			server.publish(IServer.PUBLISH_AUTO, monitor);
		}
		
		try {
			String[] equinoxProgramArgs = equinoxConfiguration.getProgramArguments();
			File tempFile = File.createTempFile("equinoxProg", "args");
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(tempFile));
			try {
				oos.writeObject(equinoxProgramArgs);
				oos.writeObject(new FrameworkLocationCalculator().getFrameworkLocations());
			} finally {
				oos.close();
			}
			ILaunchConfigurationWorkingCopy configCopy = configuration.getWorkingCopy();

			setVmArgs(monitor, configCopy, tempFile);

			configCopy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_PROGRAM_ARGUMENTS, "start");
			configCopy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_DEFAULT_CLASSPATH, false);
			addBootstrapClassPath(configCopy);
			
			new TomcatLaunchConfigurationDelegate().launch(configCopy, mode,
					launch, monitor);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	@SuppressWarnings("restriction")
	private void setVmArgs(IProgressMonitor monitor, ILaunchConfigurationWorkingCopy configCopy, File tempFile) throws CoreException {
		Server server = (Server) ServerUtil.getServer(configCopy);
		ILaunchConfiguration lc = server.getLaunchConfiguration(true, monitor);
		String vmArgsInServerConfig = lc.getAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, "");
		String vmArgs = configCopy.getAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, "");
		configCopy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, vmArgs + " " + vmArgsInServerConfig + " -Dorg.eclipse.pde.http.programArgs.file=\"" + tempFile.toString() +"\"");
	}

	private void addBootstrapClassPath(ILaunchConfigurationWorkingCopy configCopy) throws CoreException {
		Server server = (Server) ServerUtil.getServer(configCopy);
		
		String bootstrapJar = server.getRuntime().getLocation().append("bin")
				.append("bootstrap.jar").toString();
		List attribute = new ArrayList(configCopy
						.getAttribute(IJavaLaunchConfigurationConstants.ATTR_CLASSPATH,
								Collections.EMPTY_LIST));
		attribute
		.add(
				MessageFormat.format("<?xml version=\"1.0\" encoding=\"UTF-8\"?><runtimeClasspathEntry externalArchive=\"{0}\" path=\"3\" type=\"2\"/>",bootstrapJar));
		configCopy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_CLASSPATH, attribute);
	}

	VMRunnerConfiguration equinoxConfiguration;
	public IVMRunner getVMRunner(ILaunchConfiguration configuration, String mode)
			throws CoreException {
		// IVMInstall launcher = VMHelper.createLauncher(configuration);
		return new IVMRunner() {

			public void run(VMRunnerConfiguration configuration,
					ILaunch launch, IProgressMonitor monitor)
					throws CoreException {
				equinoxConfiguration = configuration;
			}

		};
	}
}
